/*
 * Licensed Materials - Property of IBM
 *
 * com.ibm.etools.c
 *
 *  Copyright IBM Corporation 2004. All Rights Reserved. 
 *
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.etools.c.importer;

import com.ibm.etools.c.CStructured;

/**
 * CStructuredDetails is a place to hold the detailed information about a
 * structure or union. There is a one-to-one relation between CStructuredDetails
 * and C_Structured. The information stored here includes the size and alignment
 * of a struct/union and the size and offset of each field. This formation is
 * used to set the size and offset of the instance in the Type Descriptor model.
 * 
 * The details information is only calculated once for each struct/union via a
 * call to findStructDetails() or findUnionDetails() in CTypeDescriptorHelper.
 * CTypeTable maintains a HashMap between a C_Structured and a
 * CStructuredDetails. Once calculated, the details information about a
 * struct/union can be later retrieved via the CTypeTable.getStructuredDetails()
 * method.
 */
public class CStructuredDetails {

    public static final String copyright = "Licensed Material - Property of IBM\n com.ibm.etools.c\n(C) Copyright IBM Corp. 2004 - All Rights Reserved.\nUS Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp."; //$NON-NLS-1$

    private CStructured _astructured;

    private int _numberOfFields;

    private int[] _fieldOffsets;

    private int[] _fieldSizes;

    private int _size;

    private int _alignment;

    /**
     * Create a CStructuredDetails object for a given struct or union
     */
    public CStructuredDetails(CStructured astructured) {
        _astructured = astructured;
        _numberOfFields = astructured.getContains().size();
        _fieldOffsets = new int[_numberOfFields];
        _fieldSizes = new int[_numberOfFields];
        _size = 0;
        _alignment = 0;
    }

    public int numberOfFields() {
        return _numberOfFields;
    }

    public void setFieldOffset(int index, int offset) {
        if (index >= 0 && index < _numberOfFields)
            _fieldOffsets[index] = offset;
    }

    public void setFieldSize(int index, int size) {
        if (index >= 0 && index < _numberOfFields)
            _fieldSizes[index] = size;
    }

    public void setSize(int size) {
        _size = size;
    }

    public void setAlignment(int alignment) {
        _alignment = alignment;
    }

    public int getFieldOffset(int index) {
        return _fieldOffsets[index];
    }

    public int getFieldSize(int index) {
        return _fieldSizes[index];
    }

    public CStructured getStructured() {
        return _astructured;
    }

    public int getSize() {
        return _size;
    }

    public int getAlignment() {
        return _alignment;
    }

}