// ****************************************************************
//
//   IBM Confidential
//   OCO Source Materials
//   <<com.ibm.etools.cobol>>
//   (C) Copyright IBM Corp. 2003 - All Rights Reserved.
//   US Government Users Restricted Rights - Use, duplication or disclosure
//   restricted by GSA ADP Schedule Contract with IBM Corp.
//
// ****************************************************************
package com.ibm.etools.cobol.importer;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import com.ibm.etools.cobol.plugin.CobolPlugin;
import com.ibm.etools.logging.util.Level;



/*
 * This class is used to parse through the compiler generated xml 
 * file(syntaxerrors.xml) and populate the Tasklist with the results.
 *@author Mel Fowler/Mahendra Akkina
 */

public class ParseSyntaxErrorsXMLOutput {


private boolean errorsFlag=false;
private String COMBINEDXMLFilename;
private Vector msgNumberVector;
private Vector msgLineVector;
private Vector msgColumnVector;
private Vector msgFileVector;
private Vector msgTextVector;
private Vector resourceVector;


private Vector errorMessagesVector;

public ParseSyntaxErrorsXMLOutput(String xmlFilename, String xmlCombinedFilename){
	COMBINEDXMLFilename = xmlCombinedFilename;
	
	combineXMLErrorFiles(xmlFilename, "", COMBINEDXMLFilename);

	/*
	* parse the created xml file, allowing access to the resulting vectors
	*/
	ErrorMsgs parsedOutput = new ErrorMsgs();
	parsedOutput.read(COMBINEDXMLFilename);
	msgNumberVector = parsedOutput.getCmpMsgNumberVector();
	msgLineVector = parsedOutput.getCmpMsgLineVector();
	msgColumnVector = parsedOutput.getCmpMsgColumnVector();
	msgFileVector = parsedOutput.getCmpMsgFileVector();
	msgTextVector = parsedOutput.getCmpMsgTextVector();
	resourceVector = parsedOutput.getCmpMsgResourceVector();
	
	//Log errors if any. Also populate the vector which is used to create the exception message with errors.
    logErrorMessages();
}

/**
 * Log errors if any. Also populate the vector which is used to create the exception message with errors.
 */
private void logErrorMessages() {
	//Check to see if there are errors
	if (msgNumberVector.size() > 0) {
		errorMessagesVector = new Vector();		
		String messageString;
		int lineNumberint;
		String fileLocation;
		String errorMessage;
		char severityChar = 'E';
		ErrorMessageInfo errorMessageInfo;
		setErrorFlag(msgNumberVector);
		for (int vi = 0; vi < msgTextVector.size(); vi++) {
			String sevMessage = (String)msgNumberVector.elementAt(vi);
			messageString =  sevMessage + " " +(String)msgTextVector.elementAt(vi);
			if (sevMessage.trim().length() > 1)	{			
				severityChar = sevMessage.trim().charAt(sevMessage.length() - 1);
			}
			lineNumberint = ((Integer)msgLineVector.elementAt(vi)).intValue();
			fileLocation = (String) msgFileVector.elementAt(vi);
			errorMessage = "Error (" + (vi+1) + ") = " + "Severity:" + severityChar + " Error message:" + messageString + " Line Number:" + lineNumberint + " File Location:" + fileLocation;
			CobolPlugin.getPlugin().writeMsg(Level.NONE,"*** ParseSyntaxErrorsXMLOutput.logErrorMessages(): " + errorMessage);			   
			errorMessagesVector.add(new ErrorMessageInfo(severityChar, messageString, lineNumberint, fileLocation));
		}
	}	
}

public void parseXMLOutputFile(IResource originalResource)
{	
	if (originalResource == null) {
		return;
	}
	int depth = IResource.DEPTH_ZERO;

	Vector adjMsgFileVector = new Vector();  //adjusted MsgFileVector contains resources not strings
	Vector adjResourceVector = new Vector(); //adjusted resourceVector contains resources not strings
	Vector adjMsgLineVector = new Vector();  //adjusted MsgLineVector if we use jcl resource, forced ln to 1

	/*
	* adjust the resource Vector                                            
	*         1) if ? at end of name, must have been inline source with jcl 
	*            replace the name with the jclfile name                     
	*/
	IResource resourcef2 = null;
	IResource xf2 = null;
	CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** ParseSyntaxErrorsXMLOutput.parseXMLOutputFile(IResource): Files involved = " + resourceVector);         
	for (int vi = 0; vi < resourceVector.size(); vi++) {
		String proposedFilename = (String)resourceVector.elementAt(vi);
		//try to save the filename if it is a relative path
		if (proposedFilename.startsWith(".\\")) {            	 	
        	 	IPath p1 = originalResource.getLocation().removeLastSegments(1);
        	 	proposedFilename = p1.toString() + proposedFilename.substring(1,proposedFilename.length());
       	}//end trying to save the filename	 
		java.io.File xf1;
		xf1 = new java.io.File (proposedFilename);
        if (xf1.exists()) {           	
        	IPath AbsPath2 = new Path (proposedFilename);
        	String drive = AbsPath2.getDevice();
        	if (drive == null){
        		drive = "";
        	}
			IFile f2 = originalResource.getWorkspace().getRoot().getFileForLocation(AbsPath2);
       	    IFile resourceF1 = null;
        	if (!(f2==null)) {
        		resourceF1 = f2;
        		adjResourceVector.addElement(resourceF1);
			} else {
             //end of trying to save the name             		
        	   adjResourceVector.addElement(originalResource);                 
        	}
        } else {
		   adjResourceVector.addElement(originalResource);            
        }						
	} //for vi < resource vector size
	/*
	* adjust the resource msgFileVector                                     
	*/         
	for (int vi = 0; vi < msgFileVector.size(); vi++) {
		java.io.File xf1;
		String proposedFilename = (String)msgFileVector.elementAt(vi);
		//try to save the filename if it is a relative path
		if (proposedFilename.startsWith(".\\")) {            	 	
    	 	IPath p1 = originalResource.getLocation().removeLastSegments(1);
    	 	proposedFilename = p1.toString() + proposedFilename.substring(1,proposedFilename.length());
        }//end trying to save the filename 			
		xf1 = new java.io.File (proposedFilename); 
        if (xf1.exists())
        {
        	IPath AbsPath2 = new Path (proposedFilename);
			IFile f2 = originalResource.getWorkspace().getRoot().getFileForLocation(AbsPath2);													  
        	IFile resourceF1 = null;
        	if (!(f2==null)) {
        		adjMsgFileVector.addElement(f2);
				adjMsgLineVector.addElement((Integer)msgLineVector.elementAt(vi));
        	} else {
             //end of trying to save the name             		
			    adjMsgFileVector.addElement(originalResource);
		        adjMsgLineVector.addElement(determineLineNumberInIncludeChain());
        	}
        	
        }//if we were able to even make a File handle out of it
        else
        {
	        adjMsgLineVector.addElement(determineLineNumberInIncludeChain());
		    adjMsgFileVector.addElement(originalResource);            
        }							  
	 } //for vi < resource vector size

	/*
	/delete resources currently in tasklist                      
	*/
	for (int vi = 0; vi < adjResourceVector.size(); vi++) {
		IResource resource1 = (IResource)adjResourceVector.elementAt(vi);
		resource1 = chkFileExtension(resource1, originalResource);
		try {
			CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** ParseSyntaxErrorsXMLOutput.parseXMLOutputFile(IResource): delete resources currently in tasklist: resource="+resource1.getFullPath());         				
			resource1.deleteMarkers(IMarker.PROBLEM, true, depth);
		}
		catch (CoreException e) {
				CobolPlugin.getPlugin().writeMsg(Level.SEVERE,"*** ParseSyntaxErrorsXMLOutput.parseXMLOutputFile(IResource): CoreException Occured while deleting resources currently in tasklist ", e);         			   
		}
	}
	/*
	*mef@stl we read the errors file, now fill in
	*mef@stl markers                             
	*/
	CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** ParseSyntaxErrorsXMLOutput.parseXMLOutputFile(IResource): Number of errors = " + msgTextVector.size());
	
	for (int vi = 0; vi < msgTextVector.size(); vi++) {
		try{
		   IResource zresource1;
		   zresource1 = (IResource)adjMsgFileVector.elementAt(vi);
		      zresource1 = chkFileExtension(zresource1, originalResource);
		   int zLineNumberint = ((Integer)adjMsgLineVector.elementAt(vi)).intValue();
		   String zmessageString = (String)msgNumberVector.elementAt(vi) + " " +(String)msgTextVector.elementAt(vi);		   
				IMarker marker = zresource1.createMarker(IMarker.PROBLEM);
				marker.setAttribute(IMarker.MESSAGE, zmessageString);
				marker.setAttribute(IMarker.LINE_NUMBER, zLineNumberint);
			/*
			*Look at the severity of the message and display the message accordingly
			*/
			String sevMsg = (String) msgNumberVector.elementAt(vi);
			sevMsg = sevMsg.trim();
			char sevChar = 'E';
			if (sevMsg.length() > 1) {
				sevChar = sevMsg.charAt(sevMsg.length() - 1);
			}
			if (sevChar == 'I') {
				marker.setAttribute(
					IMarker.SEVERITY,
					IMarker.SEVERITY_INFO);
			} else if (sevChar == 'W') {
				marker.setAttribute(
					IMarker.SEVERITY,
					IMarker.SEVERITY_WARNING);
			} else {
				//all others get a red x
				marker.setAttribute(
					IMarker.SEVERITY,
					IMarker.SEVERITY_ERROR);
			}					
		} catch (CoreException e) {
			CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** ParseSyntaxErrorsXMLOutput.parseXMLOutputFile(IResource): CoreException: Unable to create markers. Code should never come here");			
		} //catch
	} //for

}

/*Determine the line number that an include file was included from if the include file is not in
 * the Workspace.  The line number should be the line number of the last include in the chain
 * that was in the workspace.
 */
 public Integer determineLineNumberInIncludeChain()
 {
 	//This is just staged in.  Figure out the chain when there is more time!
    int TempInt = 1;
    Integer ln1LineNumberIntObject = new Integer(TempInt);
 	
   return ln1LineNumberIntObject;
 }


/*If at first we thought the include file was there, but after the 
 * compilation we cannot find it, the file type may have gotten uppercased
 * or lowercased.  Flip the casing, and then check again.
 * Just brute force my way through the names 
 * 00 lowercase.lowercase
 * 01 lowercase.uppercase
 * 10 uppercase.lowercase
 * 11 uppercase.uppercase
 */
 public IResource chkFileExtension(IResource resourceParm, IResource originalResource)
 {
 	 IResource resource = resourceParm;
 	 boolean successfulFindInWorkspace = false;
     IResource resource1 = resource;	
     IResource lowerCaseResource = null;
     IResource upperCaseResource = null;
     
     if (resource1.exists()){
            successfulFindInWorkspace = true;
            resource = resource1;
     }
     //********
     //* 00   *
     //********
   if (!successfulFindInWorkspace)
     {//try lowercasing the entire last segment
        resource1 = resource;
        IPath p1 = resource1.getLocation();
        String lastseg = p1.lastSegment();
        p1 = p1.removeLastSegments(1);
        p1 = p1.append(lastseg.toLowerCase());
		IPath AbsPath2 = new Path (p1.toString());
		IFile f2 = originalResource.getWorkspace().getRoot().getFileForLocation(AbsPath2);		
		//IFile f2 = PBPlugin.getDefault().getS390Workspace().getRoot().getFileForLocation(AbsPath2); 													  
	    resource1 = (IResource)f2;
	    lowerCaseResource = resource1;
        if (resource1.exists()){
			successfulFindInWorkspace = true;
			resource = resource1;
		 }
     }
     //********
     //* 11   *
     //********

   if (!successfulFindInWorkspace)
     {//try lowercasing the entire last segment
        resource1 = resource;
        IPath p1 = resource1.getLocation();
        String lastseg = p1.lastSegment();
        p1 = p1.removeLastSegments(1);
        p1 = p1.append(lastseg.toUpperCase());
		IPath AbsPath2 = new Path (p1.toString());
		IFile f2 = originalResource.getWorkspace().getRoot().getFileForLocation(AbsPath2);
//		IFile f2 = PBPlugin.getDefault().getS390Workspace().getRoot().getFileForLocation(AbsPath2); 													  
	    resource1 = (IResource)f2;
	    upperCaseResource = resource1;
        if (resource1.exists()){
			successfulFindInWorkspace = true;
			resource = resource1;
		 }
     }
     //********
     //* 10   *
     //********

     if (!successfulFindInWorkspace)
     {//try flipping the Uppercase resoruce to have a lowercase extension
        resource1 = upperCaseResource;
        IPath p1 = resource1.getLocation();
	  	if (!(resource1.getFileExtension() == null))
	  	{
	  		//if the resource has a file extension
	  		String ext1 = resource1.getFileExtension().toLowerCase();
            p1 = p1.removeFileExtension();
		    p1 = p1.addFileExtension(ext1);
			IPath AbsPath2 = new Path (p1.toString());
			IFile f2 = originalResource.getWorkspace().getRoot().getFileForLocation(AbsPath2);			
//			IFile f2 = PBPlugin.getDefault().getS390Workspace().getRoot().getFileForLocation(AbsPath2); 													  
		    resource1 = (IResource)f2;
		     if (resource1.exists()){
				successfulFindInWorkspace = true;
				resource = resource1;
			 }
	  	   }
	  	   
     }
     
     //********
     //* 01   *
     //********
     
     if (!successfulFindInWorkspace)
     {//try flipping the lowercase resource to have a uppercase extension
        resource1 = lowerCaseResource;
        IPath p1 = resource1.getLocation();
	  	if (!(resource1.getFileExtension() == null))
	  	{
	  		//if the resource has a file extension
	  		String ext1 = resource1.getFileExtension().toUpperCase();
            p1 = p1.removeFileExtension();
		    p1 = p1.addFileExtension(ext1);
			IPath AbsPath2 = new Path (p1.toString());
			IFile f2 = originalResource.getWorkspace().getRoot().getFileForLocation(AbsPath2);			
//			IFile f2 = PBPlugin.getDefault().getS390Workspace().getRoot().getFileForLocation(AbsPath2); 													  
		    resource1 = (IResource)f2;
		     if (resource1.exists()){
				successfulFindInWorkspace = true;
				resource = resource1;
			 }
	  	   }
	  	   
     }
     if (!successfulFindInWorkspace)
     { //Still no luck, use the default resource    
     	resource = originalResource;
     	
     }
   return resource;
 }








/*If at first we thought the include file was there, but after the 
 * compilation we cannot find it, the file type may have gotten uppercased
 * or lowercased.  Flip the casing, and then check again.
 * 
 * First:  We looked for it using java.io.File <found it case of extension didn't seem to matter>
 * Second:  We try to make IFile out of it in the workspace (can do this if the name is in the workspace)
 * Third:  Try to use the resource (marker deletion is more case sensitive than just making an IFile out of the resource)
 */
 public IResource chkFileExtension00(IResource resource)
 {
     IResource resource1 = resource;	
     IPath p1 = resource1.getLocation();
//   	  if (!(resource1.exists())){
//	  	//if resource doesn't exist, but it looked like it did
//	  	if (!(resource1.getFileExtension() == null))
//	  	{
//	  		//if the resource has a file extension
//	  		String ext1 = resource1.getFileExtension();
//
//	            p1 = p1.removeFileExtension();
//	  		if (ext1.equals(ext1.toLowerCase()))	  		
//	  		{
//		  		ext1 = ext1.toUpperCase();
//	  		}
//	  		else
//	  		{
//		  		ext1 = ext1.toLowerCase();
//	  		}	
//	            p1 = p1.addFileExtension(ext1);
//            	IPath AbsPath2 = new Path (p1.toString());
//            	IFile f2 = PBPlugin.getDefault().getS390Workspace().getRoot().getFileForLocation(AbsPath2); 													  
//	            resource1 = (IResource)f2;
//	  	   }
//	      }
  	
   return resource1;
 }


public boolean combineXMLErrorFiles(String xmlfile1, String xmlfile2, String xmlfileout)
{
   String xmlHeaderLine1 = "<?xml"; //$NON-NLS-1$
   String xmlHeaderLine2 = "<!DOCTYPE"; //$NON-NLS-1$
   String buildTag    = "<build>"; //$NON-NLS-1$
   String endbuildTag = "</build>"; //$NON-NLS-1$
   String headerLine1 = "<?xml version=\"1.0\" standalone=\"yes\"?>"; //$NON-NLS-1$
   //String headerLine2 = "<!DOCTYPE combined SYSTEM \"combined.dtd\">"; //$NON-NLS-1$
   //mef@stl providing a dtd in DOCTYPE conflicts with saying standalone="yes"
   String headerLine2 = "<!-- ************************************************************************************ -->"; //$NON-NLS-1$
   String headerLine0 = "<!-- ************************************************************************************ -->"; //$NON-NLS-1$
	boolean wroteHeader = false;
	 //  i/o to combine xml files
	 try{

	 //FileWriter outfile;  // fix: t31726
	 OutputStreamWriter outfile;
	 BufferedWriter obuff;


	 //outfile = new FileWriter(xmlfileout);  // fix: t31726
	 outfile = new OutputStreamWriter(new FileOutputStream(xmlfileout), "UTF-8"); //$NON-NLS-1$
	 obuff   = new BufferedWriter(outfile);

	  //start file with standard xml headers

	 boolean eof = false;
	  //first get the COBOL STUFF
	   for (int j = 1; j < 3; j++)
	   {
	    FileReader infile=null;
		 BufferedReader ibuff;
       boolean foundTheXMLFile = false;
		 if (j == 1)
		   {

	         java.io.File file = new java.io.File(xmlfile1);
	      	if (file.exists()) {
		   	infile  = new FileReader(xmlfile1);
            foundTheXMLFile = true;
            }
		   }
		   else
		   {	
			 //j == 2
			 if (!(xmlfile2.trim().equals("")))
			 {
	         java.io.File file = new java.io.File(xmlfile2);
	      	if (file.exists()) {
			   	infile  = new FileReader(xmlfile2);
	            foundTheXMLFile = true;
              }
    		 }
			 else
			 {
			    foundTheXMLFile = false;
			 }
		   }
       if (foundTheXMLFile)
       {
		 ibuff   = new BufferedReader(infile);
		 while (!eof) {
				 boolean skip = false;
				 String line = ibuff.readLine();
				 if (line == null)
					{
					eof = true;
					}//if
				 else
					{
					if (!wroteHeader)
					   {
						 obuff.write(headerLine1,0,headerLine1.length());
						 obuff.newLine();
						 obuff.write(headerLine2,0,headerLine2.length());
						 obuff.newLine();
						 obuff.write(buildTag,0,buildTag.length());
						 obuff.newLine();
						 wroteHeader = true;
					   }
					line = line.trim();
					if (line.equals(buildTag))
					   {
						  skip = true;
					   }
					if (line.equals(endbuildTag))
					   {
						  skip = true;
					   }
					if (line.equals(headerLine0))
					   {
						  skip = true;
					   }
					if (line.startsWith(xmlHeaderLine1))
					   {
						  skip = true;
					   }
					if (line.startsWith(xmlHeaderLine2))
					   {
						  skip = true;
					   }
					if (!skip)
					   {
						 obuff.write(line,0,line.length());
						 obuff.newLine();
					   }
					}//else
			 }//while
			 ibuff.close();
         }//if foundTheXMLFile
			 eof = false;
			 }//for j = 1 to 2
          if (wroteHeader)
          {
			    obuff.write(endbuildTag,0,endbuildTag.length());
          }
			 obuff.close();

		 } //try
	  catch (IOException e)
		 {
            CobolPlugin.getPlugin().writeMsg(Level.SEVERE,"*** ParseSyntaxErrorsXMLOutput.combineXMLErrorFiles(String, String, String): IOException occurred", e);         		 	
		 }
  if (wroteHeader)
     return true;
  else
     return false;
}

/**
 * @param msgNumberVector
 * If the error message severity > warning, then set the flag it to TRUE
 */
private void setErrorFlag(Vector msgNumberVector) {
	for (int vi = 0; vi < msgNumberVector.size(); vi++) {
	   String sevMsg = (String) msgNumberVector.elementAt(vi);
	   sevMsg = sevMsg.trim();
	   char sevChar = 'E';
	   if (sevMsg.length() > 1) {
		   sevChar = sevMsg.charAt(sevMsg.length() - 1);
	   }
	   if (!(sevChar == 'I' || sevChar == 'W')) {
		    CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** ParseSyntaxErrorsXMLOutput.setErrorFlag(Vector): Found severe error = " + sevMsg);	   	
			errorsFlag = true;
			break;	
	   }
	} //for	
}

public boolean hasErrors(){
	return errorsFlag;
}

public Vector getErrorMessagesVector() {
	return errorMessagesVector;
}

public Vector getResourceVector() {
	return resourceVector;
}

}
